// Модуль для сериализации данных в формат YAML

// Основная функция сериализации
//
// Параметры:
//   Значение - Произвольный - данные для сериализации
//   УровеньОтступа - Число - текущий уровень отступа (по умолчанию 0)
//
// Возвращаемое значение:
//   Строка - YAML представление данных
//
Функция Сериализовать(Знач Значение, УровеньОтступа = 0) Экспорт
	
	ТипЗначения = ТипЗнч(Значение);
	
	Если ТипЗначения = Тип("Строка") Тогда
		Возврат СериализоватьСтроку(Значение);
	ИначеЕсли ТипЗначения = Тип("Число") Тогда
		Возврат СериализоватьЧисло(Значение);
	ИначеЕсли ТипЗначения = Тип("Булево") Тогда
		Возврат СериализоватьБулево(Значение);
	ИначеЕсли ТипЗначения = Тип("Массив") Тогда
		Возврат СериализоватьМассив(Значение, УровеньОтступа);
	ИначеЕсли ТипЗначения = Тип("Структура") Или ТипЗначения = Тип("Соответствие") Тогда
		Возврат СериализоватьОбъект(Значение, УровеньОтступа);
	ИначеЕсли Значение = Неопределено Тогда
		Возврат "null";
	ИначеЕсли ТипЗначения = Тип("Дата") Тогда
		// Используем JSON писателя для корректной сериализации дат
		ЗаписьJSON = Новый ЗаписьJSON;
		ЗаписьJSON.УстановитьСтроку();
		ЗаписатьJSON(ЗаписьJSON, Значение);
		Возврат ЗаписьJSON.Закрыть();
	Иначе
		// Для неизвестных типов пытаемся использовать JSON сериализацию
		Попытка
			ЗаписьJSON = Новый ЗаписьJSON;
			ЗаписьJSON.УстановитьСтроку();
			ЗаписатьJSON(ЗаписьJSON, Значение);
			Возврат ЗаписьJSON.Закрыть();
		Исключение
			// Если JSON не справился, используем строковое представление
			Возврат СериализоватьСтроку(Строка(Значение));
		КонецПопытки;
	КонецЕсли;
	
КонецФункции

// Сериализация строки
//
// Параметры:
//   Значение - Строка - строка для сериализации
//
// Возвращаемое значение:
//   Строка - YAML представление строки
//
Функция СериализоватьСтроку(Значение)
	
	// Если строка пустая
	Если Значение = "" Тогда
		Возврат """""";
	КонецЕсли;
	
	// Проверяем, нужно ли заключать в кавычки
	Если ТребуетсяЗаключениеВКавычки(Значение) Тогда
		// Используем JSON писателя для корректного экранирования строки
		ЗаписьJSON = Новый ЗаписьJSON;
		ЗаписьJSON.УстановитьСтроку();
		ЗаписатьJSON(ЗаписьJSON, Значение);
		Возврат ЗаписьJSON.Закрыть();
	Иначе
		Возврат Значение;
	КонецЕсли;
	
КонецФункции

// Сериализация числа
//
// Параметры:
//   Значение - Число - число для сериализации
//
// Возвращаемое значение:
//   Строка - YAML представление числа
//
Функция СериализоватьЧисло(Значение)
	Если Значение = 0 Тогда
		Возврат "0";
	Иначе
		Возврат Формат(Значение, "ЧРД=.;ЧГ=");
	КонецЕсли;
КонецФункции

// Сериализация булевого значения
//
// Параметры:
//   Значение - Булево - булевое значение для сериализации
//
// Возвращаемое значение:
//   Строка - YAML представление булевого значения
//
Функция СериализоватьБулево(Значение)
	Если Значение Тогда
		Возврат "true";
	Иначе
		Возврат "false";
	КонецЕсли;
КонецФункции

// Сериализация массива
//
// Параметры:
//   Массив - Массив - массив для сериализации
//   УровеньОтступа - Число - текущий уровень отступа
//
// Возвращаемое значение:
//   Строка - YAML представление массива
//
Функция СериализоватьМассив(Массив, УровеньОтступа = 0)
	
	Если Массив.Количество() = 0 Тогда
		Возврат "[]";
	КонецЕсли;
	
	Результат = "";
	Отступ = ПолучитьОтступ(УровеньОтступа);
	
	Для Каждого Элемент Из Массив Цикл
		Результат = Результат + Символы.ПС;
		
		СериализованныйЭлемент = Сериализовать(Элемент, УровеньОтступа + 1);
		МассивСтрок = СтрРазделить(СериализованныйЭлемент, Символы.ПС, Ложь);
		СериализованныйЭлемент = "";
		ОтступВложения = ПолучитьОтступ(УровеньОтступа + 1);
		Для каждого Строка Из МассивСтрок Цикл
			СериализованныйЭлемент = СериализованныйЭлемент + ?(ЗначениеЗаполнено(СериализованныйЭлемент), Символы.ПС + ОтступВложения, Отступ + "- ") + СокрЛП(Строка);
		КонецЦикла;
		// Убираем последний отступ
		СериализованныйЭлемент = СокрП(СериализованныйЭлемент);
		Результат = Результат + СериализованныйЭлемент;

	КонецЦикла;
	
	Возврат Результат;
	
КонецФункции

// Сериализация объекта
//
// Параметры:
//   Объект - Соответствие, Структура - соответствие для сериализации
//   УровеньОтступа - Число - текущий уровень отступа
//
// Возвращаемое значение:
//   Строка - YAML представление соответствия
//
Функция СериализоватьОбъект(Объект, УровеньОтступа = 0)
	
	Если Объект.Количество() = 0 Тогда
		Возврат "{}";
	КонецЕсли;
	
	Результат = "";
	Отступ = ПолучитьОтступ(УровеньОтступа);
	
	Для Каждого КлючЗначение Из Объект Цикл
		Результат = Результат + Символы.ПС;
		
		Ключ = КлючЗначение.Ключ;
		Значение = КлючЗначение.Значение;
		
		СериализованныйКлюч = СериализоватьКлюч(Ключ);
		СериализованноеЗначение = Сериализовать(Значение, УровеньОтступа + 1);
		

			Результат = Результат + Отступ + СериализованныйКлюч + ": " + СериализованноеЗначение;
	КонецЦикла;
	
	Возврат Результат;
	
КонецФункции

// Сериализация ключа
//
// Параметры:
//   Ключ - Произвольный - ключ для сериализации
//
// Возвращаемое значение:
//   Строка - YAML представление ключа
//
Функция СериализоватьКлюч(Ключ)
	
	ТипКлюча = ТипЗнч(Ключ);
	
	Если ТипКлюча = Тип("Строка") Тогда
		// Для ключей применяем те же правила, что и для строк
		Возврат СериализоватьСтроку(Ключ);
	ИначеЕсли ТипКлюча = Тип("Число") Тогда
		Возврат СериализоватьЧисло(Ключ);
	ИначеЕсли ТипКлюча = Тип("Булево") Тогда
		Возврат СериализоватьБулево(Ключ);
	Иначе
		// Для других типов преобразуем в строку
		Возврат СериализоватьСтроку(Строка(Ключ));
	КонецЕсли;
	
КонецФункции

// Получение строки отступа
//
// Параметры:
//   УровеньОтступа - Число - уровень отступа
//
// Возвращаемое значение:
//   Строка - строка с соответствующим количеством пробелов
//
Функция ПолучитьОтступ(УровеньОтступа)
	
	Если УровеньОтступа <= 0 Тогда
		Возврат "";
	КонецЕсли;
	
	// Используем 2 пробела на каждый уровень отступа
	КоличествоПробелов = УровеньОтступа * 2;
	
	Результат = "";
	Пока СтрДлина(Результат) < КоличествоПробелов Цикл
		Результат = Результат + " ";
	КонецЦикла;
	
	Возврат Результат;
	
КонецФункции

// Проверка, является ли строка числом
//
// Параметры:
//   Значение - Строка - строка для проверки
//
// Возвращаемое значение:
//   Булево - Истина, если строка представляет число
//
Функция ЭтоЧисло(Значение)
	
	Если Значение = "" Тогда
		Возврат Ложь;
	КонецЕсли;
	
	// Убираем пробелы по краям
	ОчищенноеЗначение = СокрЛП(Значение);
	
	// Проверяем регулярным выражением на простое число
	Если СтрДлина(ОчищенноеЗначение) = 0 Тогда
		Возврат Ложь;
	КонецЕсли;
	
	// Простая проверка: число может содержать цифры, знак в начале и одну точку
	РазрешенныеСимволы = "0123456789.-+";
	КоличествоТочек = 0;
	КоличествоЗнаков = 0;
	
	Для Позиция = 1 По СтрДлина(ОчищенноеЗначение) Цикл
		Символ = Сред(ОчищенноеЗначение, Позиция, 1);
		
		Если СтрНайти(РазрешенныеСимволы, Символ) = 0 Тогда
			Возврат Ложь;
		КонецЕсли;
		
		Если Символ = "." Тогда
			КоличествоТочек = КоличествоТочек + 1;
		КонецЕсли;
		
		Если Символ = "-" ИЛИ Символ = "+" Тогда
			КоличествоЗнаков = КоличествоЗнаков + 1;
			// Знак может быть только в начале
			Если Позиция > 1 Тогда
				Возврат Ложь;
			КонецЕсли;
		КонецЕсли;
	КонецЦикла;
	
	// Не может быть больше одной точки или одного знака
	Возврат КоличествоТочек <= 1 И КоличествоЗнаков <= 1;
	
КонецФункции

// Проверка, требуется ли заключение строки в кавычки
//
// Параметры:
//   Значение - Строка - строка для проверки
//
// Возвращаемое значение:
//   Булево - Истина, если строку нужно заключить в кавычки
//
Функция ТребуетсяЗаключениеВКавычки(Значение)
	
	// Пустая строка должна быть в кавычках
	Если Значение = "" Тогда
		Возврат Истина;
	КонецЕсли;
	
	// Строки, которые могут интерпретироваться как специальные значения YAML
	СпециальныеЗначения = Новый Массив;
	СпециальныеЗначения.Добавить("true");
	СпециальныеЗначения.Добавить("false");
	СпециальныеЗначения.Добавить("null");
	СпециальныеЗначения.Добавить("~");
	СпециальныеЗначения.Добавить("yes");
	СпециальныеЗначения.Добавить("no");
	СпециальныеЗначения.Добавить("on");
	СпециальныеЗначения.Добавить("off");
	
	НижнийРегистр = НРег(Значение);
	Для Каждого СпециальноеЗначение Из СпециальныеЗначения Цикл
		Если НижнийРегистр = СпециальноеЗначение Тогда
			Возврат Истина;
		КонецЕсли;
	КонецЦикла;
	
	// Проверяем, является ли строка числом
	Если ЭтоЧисло(Значение) Тогда
		Возврат Истина; // Если это число, заключаем в кавычки
	КонецЕсли;
	
	// Проверяем специальные символы YAML
	СпециальныеСимволы = ":#@`|>{}[]!%&*";
	Для Инд = 1 По СтрДлина(СпециальныеСимволы) Цикл
		Символ = Сред(СпециальныеСимволы, Инд, 1);
		Если СтрНайти(Значение, Символ) > 0 Тогда
			Возврат Истина;
		КонецЕсли;
	КонецЦикла;
	
	// Проверяем, начинается ли с пробела или заканчивается пробелом
	Если Лев(Значение, 1) = " " ИЛИ Прав(Значение, 1) = " " Тогда
		Возврат Истина;
	КонецЕсли;
	
	// Проверяем на наличие переносов строк, табов и кавычек
	Если СтрНайти(Значение, Символы.ПС) > 0 ИЛИ СтрНайти(Значение, Символы.ВК) > 0 
		ИЛИ СтрНайти(Значение, Символы.ВТаб) > 0 ИЛИ СтрНайти(Значение, """") > 0 Тогда
		Возврат Истина;
	КонецЕсли;
	
	Возврат Ложь;
	
КонецФункции
