// Тесты для проверки форматирования сериализации YAML

#Использовать ".."
#Использовать asserts

&ТестовыйНабор
Процедура ПриСозданииОбъекта() Экспорт
    // Инициализация
КонецПроцедуры

&Тест
Процедура ТестФорматированияОтступов() Экспорт
    // Дано
    Парсер = Новый ПарсерYaml();
    
    // Создаем простую структуру и записываем ее в YAML
    ТестоваяСтруктура = Новый Структура;
    ТестоваяСтруктура.Вставить("ключ", "значение");
    ТестоваяСтруктура.Вставить("вложенный", Новый Структура("поле1, поле2", "значение1", "значение2"));
    
    // Когда
    СтрокаYaml = Парсер.ЗаписатьYaml(ТестоваяСтруктура);
    
    // Тогда - проверяем форматирование
    Ожидаем.Что(СтрокаYaml).Не_().ЭтоНеопределено();
    
    // Проверяем наличие ключей и отступов
    Ожидаем.Что(СтрНайти(СтрокаYaml, "ключ:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "вложенный:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "поле1:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "поле2:")).Больше(0);
КонецПроцедуры

&Тест
Процедура ТестФорматированияМассивов() Экспорт
    // Дано
    Парсер = Новый ПарсерYaml();
    
    // Создаем массив и записываем его в YAML
    МассивЗначений = Новый Массив;
    МассивЗначений.Добавить("элемент1");
    МассивЗначений.Добавить("элемент2");
    МассивЗначений.Добавить("элемент3");
    
    // Когда
    СтрокаYaml = Парсер.ЗаписатьYaml(МассивЗначений);
    
    // Тогда - проверяем форматирование массива
    Ожидаем.Что(СтрокаYaml).Не_().ЭтоНеопределено();
    
    // Проверяем наличие элементов массива и их форматирование
    Ожидаем.Что(СтрНайти(СтрокаYaml, "элемент1")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "элемент2")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "элемент3")).Больше(0);
КонецПроцедуры

&Тест
Процедура ТестФорматированияГлубокоВложенныхСтруктур() Экспорт
    // Дано
    Парсер = Новый ПарсерYaml();
    
    // Создаем глубоко вложенную структуру
    Уровень3 = Новый Структура("поле3", "значение3");
    Уровень2 = Новый Структура("поле2", Уровень3);
    Уровень1 = Новый Структура("поле1", Уровень2);
    КорневаяСтруктура = Новый Структура("корень", Уровень1);
    
    // Когда
    СтрокаYaml = Парсер.ЗаписатьYaml(КорневаяСтруктура);
    
    // Тогда - проверяем форматирование вложенной структуры
    Ожидаем.Что(СтрокаYaml).Не_().ЭтоНеопределено();
    
    // Проверяем наличие ключей и отступов для каждого уровня
    Ожидаем.Что(СтрНайти(СтрокаYaml, "корень:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "поле1:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "поле2:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "поле3:")).Больше(0);
КонецПроцедуры

&Тест
Процедура ТестФорматированияСложныхМассивов() Экспорт
    // Дано
    Парсер = Новый ПарсерYaml();
    
    // Создаем массив объектов
    Объект1 = Новый Структура("имя, возраст", "Иван", 25);
    Объект2 = Новый Структура("имя, возраст", "Мария", 30);
    
    МассивОбъектов = Новый Массив;
    МассивОбъектов.Добавить(Объект1);
    МассивОбъектов.Добавить(Объект2);
    
    // Когда
    СтрокаYaml = Парсер.ЗаписатьYaml(МассивОбъектов);
    
    // Тогда - проверяем форматирование массива объектов
    Ожидаем.Что(СтрокаYaml).Не_().ЭтоНеопределено();
    
    // Проверяем наличие корректных отступов и структуру
    Ожидаем.Что(СтрНайти(СтрокаYaml, "имя:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "возраст:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "Иван")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "Мария")).Больше(0);
КонецПроцедуры

&Тест
Процедура ТестОбработкиПустыхЗначений() Экспорт
    // Дано
    Парсер = Новый ПарсерYaml();
    
    // Создаем структуру с различными пустыми значениями
    ТестоваяСтруктура = Новый Структура;
    ТестоваяСтруктура.Вставить("пустаяСтрока", "");
    ТестоваяСтруктура.Вставить("неопределено", Неопределено);
    ТестоваяСтруктура.Вставить("пустойМассив", Новый Массив);
    ТестоваяСтруктура.Вставить("пустаяСтруктура", Новый Структура);
    
    // Когда
    СтрокаYaml = Парсер.ЗаписатьYaml(ТестоваяСтруктура);
    
    // Тогда - проверяем обработку пустых значений
    Ожидаем.Что(СтрокаYaml).Не_().ЭтоНеопределено();
    
    // Проверяем корректное форматирование пустых значений
    Ожидаем.Что(СтрНайти(СтрокаYaml, "пустаяСтрока:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "неопределено:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "пустойМассив:")).Больше(0);
    Ожидаем.Что(СтрНайти(СтрокаYaml, "пустаяСтруктура:")).Больше(0);
КонецПроцедуры

&Тест
Процедура ТестМассивСОднимЭлементом() Экспорт
	// Подготовка
	МассивСОднимЭлементом = Новый Массив;
	МассивСОднимЭлементом.Добавить("backend");
	
	Парсер = Новый ПарсерYaml();
	
	// Действие
	РезультатСериализации = Парсер.ЗаписатьYaml(МассивСОднимЭлементом);
	
	// Проверка
	Ожидаем.Что(СокрЛП(РезультатСериализации)).Равно("- backend");
КонецПроцедуры

&Тест
Процедура ТестМассивСОднимВложеннымОбъектом() Экспорт
	// Подготовка
	ВложенныйОбъект = Новый Структура;
	ВложенныйОбъект.Вставить("external", Истина);
	
	МассивСОднимЭлементом = Новый Массив;
	МассивСОднимЭлементом.Добавить(ВложенныйОбъект);
	
	Парсер = Новый ПарсерYaml();
	
	// Действие
	РезультатСериализации = Парсер.ЗаписатьYaml(МассивСОднимЭлементом);
	
	// Проверка
	ОжидаемыйРезультат = 
	"- external: true";
	
	Ожидаем.Что(СокрЛП(РезультатСериализации)).Равно(ОжидаемыйРезультат);
КонецПроцедуры

&Тест
Процедура ТестОтступыВКоллекциях() Экспорт
	// Подготовка
	Структура = Новый Структура;
	Структура.Вставить("volumes", Новый Структура);
	
	ВложенныйМассив = Новый Массив;
	ВложенныйМассив.Добавить("radarr_config: /config");
	
	Структура.volumes.Вставить("элементы", ВложенныйМассив);
	
	Парсер = Новый ПарсерYaml();
	
	// Действие
	РезультатСериализации = Парсер.ЗаписатьYaml(Структура);
	
	// Проверка
	Ожидаем.Что(РезультатСериализации).Содержит("volumes:");
	Ожидаем.Что(РезультатСериализации).Содержит("элементы:");
	Ожидаем.Что(РезультатСериализации).Содержит("- ""radarr_config: /config""");
КонецПроцедуры

&Тест
Процедура ТестМассивОбъектовСОднимЭлементом() Экспорт
	// Подготовка
	МассивОбъектов = Новый Массив;
	
	Объект1 = Новый Структура;
	Объект1.Вставить("networks", Новый Массив);
	Объект1.networks.Добавить("backend");
	
	МассивОбъектов.Добавить(Объект1);
	
	Парсер = Новый ПарсерYaml();
	
	// Действие
	РезультатСериализации = Парсер.ЗаписатьYaml(МассивОбъектов);
	
	// Проверка
	Ожидаем.Что(РезультатСериализации).Содержит("networks:");
	Ожидаем.Что(РезультатСериализации).Содержит("- backend");
КонецПроцедуры
